#!/usr/local/bin/perl
#
# Script to carry out the Fisher's Tests generated by fisher_generator_allgenes.pl, using an R interpretor. The presence of free statistics package R on the drive is required to run this script, and the Statistics::R perl module to be installed. This should be installed into the /lib subdirectory of your Perl installation.
# If a one-tailed test has been requested in the command line for the Fisher's Test Generator, it must also be specified in the command line of this script. Use the word 'less' or 'greater', as in the generator script (without the quotation marks).
# R can be obtained from http://www.r-project.org/
# Statistics::R module can be obtained from http://cpan.uwinnipeg.ca/htdocs/Statistics-R/Statistics/R.html
# Statistics::R module by M. P. Graciliano (gm@virtuasites.br)
# There is a known bug in the Statistics::R module which can cause this script to freeze during its execution of the Fisher's tests for the clusters. This, along with a working resolution, is documented at CPAN, bug report 11918 (http://rt.cpan.org/NoAuth/Bug.html?id=11918)
# Acknowledgements to the Perl Monks (http://www.perlmonks.com) for advice on this script.
# Ian Grieve
# 08/08/05

use strict;
use Statistics::R ;

my $one_tailed = $ARGV[0];
$ARGV[0] = $one_tailed;
print $one_tailed;

my @R_output_store = "";
my $R_line0 = "";
my $R_line1 = "";
my $R_line2 = "";
my $R_line3 = "";
my $R_line4 = "";
my $R_line5 = "";
my $R_line6 = "";
my $add_whitespace = " ";
my $add_newline = "\n";
my $R_comment_out = "## ";
my $R_quotation_start = "q'";
my $R_dblquotation_start = "fisher_out = ";
my $R_dblquotation_end = "";
my $R_quotation_end = "'";
my $R_send_0 = "";
my $R_send_1 = "";
my $R_send_2 = "";
my $R_send_3 = " \n print (fisher_out)";
my $R_send = "";

my $R = Statistics::R->new() ;

#my $debug_out = 'fisher_debugout.txt';
#open (DEBUG, ">$debug_out");

our $R_input_script = 'fishertests.r';
open (R_IN, $R_input_script) || die print "Fisher's Testing Script cannot open $R_input_script";
$/ = "##";
my @R_input = <R_IN>;
shift @R_input;

close (R_IN);
# unlink ('fishertests.r') || die print "Fisher's Testing Script cannot unlink fishertests.r";

our $R_output_debug = 'fisher_R_output.txt';
open (R_OUT, ">$R_output_debug") || die print "Fisher's Testing Script cannot open $R_output_debug";

$R->startR ; 

sub run_r_tests
{
    foreach my $cluster (@R_input)
    {
    $cluster =~ /(Fisher's Exact Test for Cluster \d+ \([^\)]*\))/;
    $R_line0 = $1;
    $cluster =~ /(Cluster\d+ <-)/;
    $R_line1 = ($1.$add_whitespace);
    $cluster =~ /(matrix\(c\(\d+, \d+, \d+, \d+\),)/;
    $R_line2 = ($1.$add_whitespace);
    $cluster =~ /(nr = 2,)/;
    $R_line3 = ($1.$add_whitespace);
    $cluster =~ /(dimnames = list\(Annotated = c\("DE", "Non-DE"\),)/;
    $R_line4 = ($1.$add_whitespace);
    $cluster =~ /(NotAnnotated = c\("DE", "Non-DE"\)\)\))/;
    $R_line5 = $1;

       if ($one_tailed =~ /greater/)
       {
          $cluster =~ /(fisher.test\(Cluster\d+, alternative = "greater"\))/;
          $R_line6 = $1;
       }

       elsif ($one_tailed =~ /less/)
       {
          $cluster =~ /(fisher.test\(Cluster\d+, alternative = "less"\))/;
          $R_line6 = $1;
       }

       else
       {
          $cluster =~ /(fisher.test\(Cluster\d+\))/;
          $R_line6 = $1;

       }

    $R_send_0 = "$R_comment_out$R_line0";
    $R_send_1 = "$R_line1$R_line2$R_line3$R_line4$R_line5";
    $R_send_2 = "$R_dblquotation_start$R_line6$R_send_3";
    $R_send = "$R_send_1\n$R_send_2";

#    print R_OUT $R_send_0;
#    print R_OUT "\n";
#    print R_OUT $R_send_1;
#    print R_OUT "\n";
#    print R_OUT $R_send_2;
#    print R_OUT "\n\n";


my $r_send_comment = $R_send_0;
my $r_send_string = $R_send;
print $R_send;
print "\n";
$R->send (qq'$r_send_string');
push (@R_output_store, $r_send_comment);
print $r_send_comment;
print "\n";
my $ret = $R->read;
push (@R_output_store, $ret);
print $ret;
#print DEBUG "$ret\n\n";
print "\n\n";
}
}

run_r_tests;

my $cluster_name = "";
my $cluster_number = "";
my $cluster_pvalue = "";
my $cluster_confint_low = "";
my $cluster_confint_high = "";
my $cluster_confint = "";
my $cluster_confint_separator = "-";
my $cluster_oddsratio = "";
my $fisher_out_string = "";
my @fisher_out_array = "";
my $comma_separator = "* ";
my $semicolon_separator = "; ";
my @r_output_sorted = "";

# print R_OUT @R_output_store;
$/ = ##;
shift @R_output_store;

foreach my $fisher_outcome (@R_output_store)
{
    if ($fisher_outcome =~ /##[^\(]*\(([^\)]*)\)/)
	{
	    $cluster_name = $1;
#    print R_OUT "Fisher's Exact Test for";
#    print R_OUT $cluster_name;
	    $fisher_out_string = "$cluster_name$comma_separator";
	    push (@fisher_out_array, $fisher_out_string);
	}
	
	if ($one_tailed =~ /greater/)
	{
	    if ($fisher_outcome =~ /data:\s+Cluster(\d+)[^p]*p-value [=|<] (\d+.?\d*e?-?\d*)\nalternative hypothesis: true odds ratio is greater than 1[^o]*onfidence interval:\n?\s*([^\s]*)\s*([^\s]*)[^o]*odds ratio\n?\s*(\d+.?\d*)/s)
	    {
		$cluster_number = $1;
		$cluster_pvalue = $2;
		$cluster_confint_low = $3;
		$cluster_confint_high = $4;
		$cluster_oddsratio = $5;
		$fisher_out_string = "$cluster_number$comma_separator$cluster_pvalue$comma_separator$cluster_confint_low$cluster_confint_separator$cluster_confint_high$comma_separator$cluster_oddsratio$semicolon_separator";
		push (@fisher_out_array, $fisher_out_string);    
	    }
	}

	elsif ($one_tailed =~ /less/)
	{
	    if ($fisher_outcome =~ /data:\s+Cluster(\d+)[^p]*p-value [=|<] (\d+.?\d*e?-?\d*)\nalternative hypothesis: true odds ratio is less than 1[^o]*onfidence interval:\n?\s*([^\s]*)\s*([^\s]*)[^o]*odds ratio\n?\s*(\d+.?\d*)/s)
	    {
		$cluster_number = $1;
		$cluster_pvalue = $2;
		$cluster_confint_low = $3;
		$cluster_confint_high = $4;
		$cluster_oddsratio = $5;
		$fisher_out_string = "$cluster_number$comma_separator$cluster_pvalue$comma_separator$cluster_confint_low$cluster_confint_separator$cluster_confint_high$comma_separator$cluster_oddsratio$semicolon_separator";
		push (@fisher_out_array, $fisher_out_string);    
	    }
	}
	
	elsif ($fisher_outcome =~ /data:\s+Cluster(\d+)[^p]*p-value [=|<] (\d+.?\d*e?-?\d*)\nalternative hypothesis: true odds ratio is not equal to 1[^o]*onfidence interval:\n?\s*([^\s]*)\s*([^\s]*)[^o]*odds ratio\n?\s*(\d+.?\d*)/s)
	{
	    $cluster_number = $1;
	    $cluster_pvalue = $2;
	    $cluster_confint_low = $3;
	    $cluster_confint_high = $4;
	    $cluster_oddsratio = $5;
	    $fisher_out_string = "$cluster_number$comma_separator$cluster_pvalue$comma_separator$cluster_confint_low$cluster_confint_separator$cluster_confint_high$comma_separator$cluster_oddsratio$semicolon_separator";
	    push (@fisher_out_array, $fisher_out_string);    
	}
    }
    
    my $r_array_tempstring = "@fisher_out_array";
    
    my @r_output_resplit = split (/;/, $r_array_tempstring);
    
    @r_output_sorted = sort { (split /\* /, $a)[2] <=> (split /\* /, $b)[2] } @r_output_resplit;
    foreach my $sorted_posoc_cluster (@r_output_sorted)
    {
	
	if ($sorted_posoc_cluster =~ / ([^\*]*)\*\s+(\d+)\*\s+(\d+.?\d*e?-?\d*)\*\s+([^-]*-[^-]*)\*\s+(\d+.?\d*)/)
	{
	    $cluster_number = $2;
	    $cluster_name = $1;
	    $cluster_pvalue = $3;
	    $cluster_confint = $4;
	    $cluster_oddsratio = $5;
	    
	    $cluster_name =~ s/^\s+//;
	    print R_OUT "##";
	    print R_OUT "\n";
	    print R_OUT $cluster_name;
	    print R_OUT "\t";
	    print R_OUT "Cluster: ";
	    print R_OUT $cluster_number;
	    print R_OUT "\n";
	    
	    if (($cluster_pvalue <= 0.05) && ($cluster_oddsratio < 1))
	    {
		print R_OUT "P-value = ";
		print R_OUT $cluster_pvalue;
		print R_OUT "\n";
		print R_OUT "The Differentially Expressed genes are significantly under-represented in Cluster ";
		print R_OUT $cluster_number;
		print R_OUT " at the 5% level of significance.";
		print R_OUT "\n\n";
	    }
	    
	    elsif (($cluster_pvalue <= 0.05) && ($cluster_oddsratio > 1))
	    {
		print R_OUT "P-value = ";
		print R_OUT $cluster_pvalue;
		print R_OUT "\n";
		print R_OUT "The Differentially Expressed genes are significantly over-represented in Cluster ";
		print R_OUT $cluster_number;
		print R_OUT " at the 5% level of significance.";
		print R_OUT "\n\n";
	    }
	    
	    elsif ($cluster_pvalue > 0.05)
	    {
		print R_OUT "P-value = ";
		print R_OUT $cluster_pvalue;
		print R_OUT "\n\n";
	    }
	    
	    
	}
    }
    
    $R->stopR();
    close (R_IN);
    close (R_OUT);
    
